type DashboardColorScheme = 'light' | 'dark' | 'light dark';
export interface DatabricksDashboardOptions {
    /**
     * The workspace ID where the dashboard is located.
     *
     * @example 1234567890123456
     */
    workspaceId: string;
    /**
     * The instance URL of the Databricks workspace.
     *
     * @example https://my-databricks-instance
     */
    instanceUrl: string;
    /**
     * The container element to render the dashboard in.
     */
    container: HTMLElement;
    /**
     * The ID of the dashboard to render.
     *
     * @example https://my-databricks-instance/dashboardsv3/<dashboardId>/published
     */
    dashboardId: string;
    /**
     * The token generated by Databricks.
     *
     * ⚠️ CAUTION: This token is accessible by viewers of this page. Please
     * consider:
     * - **Restrict the scope of the token appropriately.** Follow Databricks documentation
     *   to ensure the scope shared in the browser only allows access to the
     *   current dashboard.
     */
    token: string;
    /**
     * Utility function that sets a new token for the dashboard.
     * Use this to maintain access to long-running dashboards without interrupting the user experience.
     *
     * @returns A new token generated by Databricks.
     */
    getNewToken?: () => Promise<string>;
    /**
     * Specifies the color scheme for the embedded dashboard.
     *
     * - "light": Forces light mode
     * - "dark": Forces dark mode
     * - "light dark": Allows the dashboard to automatically match the user's system/browser preference
     *
     * This value maps to the CSS `color-scheme` property:
     * https://developer.mozilla.org/en-US/docs/Web/CSS/color-scheme
     */
    colorScheme?: DashboardColorScheme;
}
/**
 * A Databricks dashboard that can be embedded in a web page.
 * @example
 * ```typescript
 * const dashboard = new DatabricksDashboard({
 *   workspaceId: "1234567890123456",
 *   instanceUrl: "https://my-databricks-instance.com",
 *   container: document.getElementById("dashboard-container"),
 *   dashboardId: "abcdedf123456",
 *   token: tokenThatCanBeSeenByViewers,
 *   getNewToken: getNewTokenFunction, // Retrieve a new token from your server.
 * }).initialize();
 * ```
 */
export declare class DatabricksDashboard {
    readonly container: HTMLElement;
    readonly dashboardId: string;
    readonly instanceUrl: string;
    readonly workspaceId: string;
    private getNewToken;
    private iframe;
    private isDestroyed;
    private refreshTimeoutId;
    private token;
    private isIframeLoaded;
    private colorScheme;
    constructor({ workspaceId, instanceUrl, container, dashboardId, token, getNewToken, colorScheme, }: DatabricksDashboardOptions);
    initialize(): void;
    destroy(): void;
    private initialzeIframe;
    private initializeRefreshTokenTimer;
    private displayError;
    private handleMessages;
}
export {};
//# sourceMappingURL=DatabricksDashboard.d.ts.map