import _defineProperty from '@babel/runtime/helpers/defineProperty';

let ErrorCode = /*#__PURE__*/function (ErrorCode) {
  ErrorCode["INSTANCE_DESTROYED"] = "INSTANCE_DESTROYED";
  ErrorCode["INVALID_TOKEN_FORMAT"] = "INVALID_TOKEN_FORMAT";
  ErrorCode["IFRAME_NOT_LOADED_TOKEN_EXPIRED"] = "IFRAME_NOT_LOADED_TOKEN_EXPIRED";
  ErrorCode["TOKEN_EXPIRED_NO_REFRESH_METHOD"] = "TOKEN_EXPIRED_NO_REFRESH_METHOD";
  return ErrorCode;
}({});
class DatabricksDashboardError extends Error {
  constructor(errorCode) {
    const message = DatabricksDashboardError.getMessage(errorCode);
    super(message);
    this.errorCode = errorCode;
    this.name = 'DatabricksDashboardError';
  }
  static getMessage(errorCode) {
    switch (errorCode) {
      case ErrorCode.INSTANCE_DESTROYED:
        return 'The DatabricksDashboard instance has already been destroyed.';
      case ErrorCode.INVALID_TOKEN_FORMAT:
        return 'The authentication token is incomplete or incorrectly formatted. It should include at least two parts separated by a dot (.).';
      case ErrorCode.IFRAME_NOT_LOADED_TOKEN_EXPIRED:
        return 'The iframe has not been fully initialized, but the token is about to expire. Please refresh the page. If the issue persists, contact the page owner to check the configuration.';
      case ErrorCode.TOKEN_EXPIRED_NO_REFRESH_METHOD:
        return 'The token is about to expire. Refresh the page to keep viewing.';
      default:
        return 'An unknown error occurred.';
    }
  }
}

const ERROR_TITLE_CLASSNAME = 'error-title';
const ERROR_STACK_CLASSNAME = 'error-stack';
const ERROR_TITLE_SELECTOR = `.${ERROR_TITLE_CLASSNAME}`;
const ERROR_STACK_SELECTOR = `.${ERROR_STACK_CLASSNAME}`;
const ERROR_TEMPLATE_HTML = `<!DOCTYPE html>
    <html lang='en'>
    <head>
      <meta charset='UTF-8'>
      <meta name='viewport' content='width=device-width, initial-scale=1.0'>
      <title>Dashboard Error</title>
      <style>
        .error {
          color: red;
          font-size: 13px;
          font-weight: bold;
          margin: 8px;
        }
        .error-title {
          font-weight: bold;
        }
      </style>
    </head>
    <body>
      <div class='error'>
      <h3 class="${ERROR_TITLE_CLASSNAME}"></h3>
      <pre class="${ERROR_STACK_CLASSNAME}"></pre></div>
    </body>
    </html>`;
function buildErrorDisplay(rawError) {
  const error = wrapError(rawError);
  const iframe = document.createElement('iframe');
  iframe.srcdoc = ERROR_TEMPLATE_HTML;
  iframe.style.width = '100%';
  iframe.style.height = '100%';
  iframe.style.border = 'none';
  iframe.onload = () => {
    const iframeDocument = iframe.contentDocument;
    if (!iframeDocument) {
      return;
    }
    const errorTitle = iframeDocument.querySelector(ERROR_TITLE_SELECTOR);
    if (errorTitle) {
      errorTitle.textContent = error.name;
    }
    const errorStack = iframeDocument.querySelector(ERROR_STACK_SELECTOR);
    if (errorStack && error.stack) {
      errorStack.textContent = error.stack;
    }
  };
  return iframe;
}
function wrapError(rawError) {
  if (rawError instanceof Error) {
    return rawError;
  }
  if (typeof rawError === 'string') {
    return new Error(rawError);
  }
  return new Error(JSON.stringify(rawError));
}

const FIVE_MIN_MS = 5 * 60 * 1000;
const WORKSPACE_URL_PARAM = 'o';
const TOKEN_URL_PARAM = 'token';
/**
 * A Databricks dashboard that can be embedded in a web page.
 * @example
 * ```typescript
 * const dashboard = new DatabricksDashboard({
 *   workspaceId: "1234567890123456",
 *   instanceUrl: "https://my-databricks-instance.com",
 *   container: document.getElementById("dashboard-container"),
 *   dashboardId: "abcdedf123456",
 *   token: tokenThatCanBeSeenByViewers,
 *   getNewToken: getNewTokenFunction, // Retrieve a new token from your server.
 * }).initialize();
 * ```
 */
class DatabricksDashboard {
  constructor(_ref) {
    let {
      workspaceId,
      instanceUrl,
      container,
      dashboardId,
      token,
      getNewToken,
      colorScheme
    } = _ref;
    _defineProperty(this, "isDestroyed", false);
    _defineProperty(this, "isIframeLoaded", false);
    // Note: we use an arrow function here to bind `this` to the instance of the
    // class This is necessary because the event listener is called in the context
    // of the window object, not the instance of the class.
    _defineProperty(this, "handleMessages", event => {
      if (new URL(event.origin).origin !== new URL(this.instanceUrl).origin) {
        return;
      }
      if (event.data.type === 'DATABRICKS_EMBED_READY') {
        this.isIframeLoaded = true;
      }
    });
    this.workspaceId = workspaceId;
    this.instanceUrl = instanceUrl;
    this.container = container;
    this.dashboardId = dashboardId;
    this.token = token;
    this.getNewToken = getNewToken;
    this.colorScheme = colorScheme;
    this.iframe = document.createElement('iframe');
  }

  // #region public
  initialize() {
    if (this.isDestroyed) {
      throw new DatabricksDashboardError(ErrorCode.INSTANCE_DESTROYED);
    }
    try {
      this.initialzeIframe();
      this.initializeRefreshTokenTimer();
    } catch (error) {
      this.displayError(error);
    }
  }
  destroy() {
    if (this.isDestroyed) {
      throw new DatabricksDashboardError(ErrorCode.INSTANCE_DESTROYED);
    }
    if (this.refreshTimeoutId) {
      window.clearTimeout(this.refreshTimeoutId);
    }
    window.removeEventListener('message', this.handleMessages);
    this.container.innerHTML = '';
    this.isDestroyed = true;
  }

  // #endregion public

  // #region Initialization

  initialzeIframe() {
    const url = new URL(this.instanceUrl);
    const path = `/embed/dashboardsv3/${this.dashboardId}`;
    url.pathname = path;
    url.searchParams.append(WORKSPACE_URL_PARAM, this.workspaceId);
    url.searchParams.append(TOKEN_URL_PARAM, this.token);

    // Add an event listener for this specific iframe to detect when it is loaded
    window.addEventListener('message', this.handleMessages);
    this.iframe.src = url.href;
    if (this.colorScheme) {
      this.iframe.style.colorScheme = this.colorScheme;
    }
    this.container.appendChild(this.iframe);
  }
  initializeRefreshTokenTimer() {
    const getNewToken = this.getNewToken;
    if (!getNewToken) {
      // eslint-disable-next-line no-console
      console.warn('The getNewToken function is not provided. The token will not be refreshed.');
    }
    try {
      // Parse the JWT token and determine the expiration time
      const token = this.token.split('.')[1];
      if (!token) {
        throw new DatabricksDashboardError(ErrorCode.INVALID_TOKEN_FORMAT);
      }
      const tokenPayload = JSON.parse(atob(token));
      const expirationTimestampMs = tokenPayload.exp * 1000;
      const timeLeftMs = expirationTimestampMs - Date.now();

      // Refresh interval is at most 5 minutes before the token expires
      const refreshIntervalMs = Math.max(0, timeLeftMs - FIVE_MIN_MS);
      const executeRefresh = async () => {
        try {
          if (!getNewToken) {
            throw new DatabricksDashboardError(ErrorCode.TOKEN_EXPIRED_NO_REFRESH_METHOD);
          }
          const newToken = await getNewToken();

          // Nothing to do if the instance is destroyed
          if (this.isDestroyed) return;
          this.token = newToken;

          // Send a message to the iframe to update the token
          if (this.isIframeLoaded) {
            var _this$iframe$contentW;
            (_this$iframe$contentW = this.iframe.contentWindow) === null || _this$iframe$contentW === void 0 || _this$iframe$contentW.postMessage({
              type: 'DATABRICKS_SET_TOKEN',
              token: newToken
            }, this.instanceUrl);
          } else {
            throw new DatabricksDashboardError(ErrorCode.IFRAME_NOT_LOADED_TOKEN_EXPIRED);
          }
          this.initializeRefreshTokenTimer();
        } catch (error) {
          this.displayError(error);
        }
      };
      this.refreshTimeoutId = window.setTimeout(executeRefresh, refreshIntervalMs);
    } catch (error) {
      this.displayError(error);
    }
  }

  // #endregion Initialization

  displayError(error) {
    const errorIframe = buildErrorDisplay(error);
    this.container.innerHTML = '';
    this.container.appendChild(errorIframe);
  }
}

export { DatabricksDashboard, DatabricksDashboardError, ErrorCode };
